package inappbilling;

import inappbilling_util.IabHelper;
import inappbilling_util.IabResult;
import inappbilling_util.Inventory;
import inappbilling_util.Purchase;

import java.util.ArrayList;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;

public class InAppBillingManager
{
	private IabHelper mHelper;
	private String base64EncodedPublicKey;
	private Activity act;

	private ArrayList<InAppBillingItem> iabItems;
	private Inventory inventory;

	OnIabSetupFinishedListener mListener;
	OnPurchaseFinishedListener mListenerPurchaseFinished;
	OnConsumeFinishedListener mListenerConsumeFinished;

	Boolean showThankYouDialogAfterBuying;

	public InAppBillingManager(Context ctx, Activity act)
	{
		this.act = act;
		this.iabItems = InAppBillingSetup.IAB_SKUS;
		this.base64EncodedPublicKey = InAppBillingSetup.IAB_base64EncodedPublicKey;

		// do the setup and ask for already purchased IAB_SKUS
		mHelper = new IabHelper(ctx, base64EncodedPublicKey);
		mHelper.startSetup(new IabHelper.OnIabSetupFinishedListener()
		{
			public void onIabSetupFinished(IabResult result)
			{
				// query which products were already purchased
				mHelper.queryInventoryAsync(mGotInventoryListener);
			}
		});
	}

	public void setIabSetupFinishedListener(OnIabSetupFinishedListener listener)
	{
		mListener = listener;
	}

	public void setPurchaseFinishedListener(OnPurchaseFinishedListener listener)
	{
		mListenerPurchaseFinished = listener;
	}

	public void setConsumeFinishedListener(OnConsumeFinishedListener listener)
	{
		mListenerConsumeFinished = listener;
	}

	IabHelper.QueryInventoryFinishedListener mGotInventoryListener = new IabHelper.QueryInventoryFinishedListener()
	{
		public void onQueryInventoryFinished(IabResult result, Inventory inv)
		{
			// Have we been disposed of in the meantime? If so, quit.
			if (mHelper == null)
				return;

			Boolean success = false;
			String errorMessage = null;

			if (result.isFailure())
			{
				errorMessage = "Failed to query inventory: " + result;
			} else
			{
				// save the inventory
				inventory = inv;

				// determine which IAB_SKUS were already bought
				for (InAppBillingItem purchasedItem : iabItems)
					purchasedItem.setPurchased(inventory.hasPurchase(purchasedItem.Id()));

				success = true;
			}

			// fire callback event
			if (mListener != null)
				mListener.onIabSetupFinished(success, errorMessage);
		}
	};

	IabHelper.OnIabPurchaseFinishedListener mPurchaseFinishedListener = new IabHelper.OnIabPurchaseFinishedListener()
	{
		public void onIabPurchaseFinished(IabResult result, Purchase purchase)
		{
			// Have we been disposed of in the meantime? If so, quit.
			if (mHelper == null)
				return;

			Boolean success = false;
			String errorMessage = null;
			InAppBillingItem purchasedItem = null;

			if (result.isFailure())
			{
				errorMessage = result + "";
			} else
			{
				// get item
				purchasedItem = getIabItemById(purchase.getSku());

				purchasedItem.setPurchased(true);

				// auto consume the item if required
				if (purchasedItem.AutoConsumeAfterPurchasing())
					consumeItem(purchasedItem);

				success = true;

				// show thank you text
				if (showThankYouDialogAfterBuying)
					alert(purchasedItem.ThankYouText());
			}

			// fire callback event
			if (mListenerPurchaseFinished != null)
				mListenerPurchaseFinished.onPurchaseFinished(success, errorMessage, purchasedItem);
		}
	};

	// Called when consumption is complete
	IabHelper.OnConsumeFinishedListener mConsumeFinishedListener = new IabHelper.OnConsumeFinishedListener()
	{
		public void onConsumeFinished(Purchase purchase, IabResult result)
		{
			// Have we been disposed of in the meantime? If so, quit.
			if (mHelper == null)
				return;

			Boolean success = false;
			String errorMessage = null;
			InAppBillingItem consumedItem = null;

			if (result.isFailure())
			{
				errorMessage = result + "";
			} else
			{
				// get item
				consumedItem = getIabItemById(purchase.getSku());

				// set item attribute purchased = false
				consumedItem.setPurchased(false);

				success = true;
			}

			// fire callback event
			if (mListenerConsumeFinished != null)
				mListenerConsumeFinished.onConsumeFinished(success, errorMessage, consumedItem);
		}
	};

	public void purchaseItem(final InAppBillingItem item, Boolean showInfoDialogBeforeBuying, Boolean showThankYouDialogAfterBuying)
	{
		final Integer request = 1;
		final String payload = "";
		this.showThankYouDialogAfterBuying = showThankYouDialogAfterBuying;

		if (showInfoDialogBeforeBuying)
		{
			// build info dialog
			AlertDialog.Builder alert = new AlertDialog.Builder(act);

			alert.setTitle(item.InfoDialogTitle());
			alert.setMessage(item.InfoDialogDescription());

			alert.setPositiveButton("Ok", new DialogInterface.OnClickListener()
			{

				@Override
				public void onClick(DialogInterface dialog, int which)
				{
					mHelper.launchPurchaseFlow(act, item.Id(), request, mPurchaseFinishedListener, payload);
				}
			});

			alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener()
			{

				@Override
				public void onClick(DialogInterface dialog, int which)
				{
					;
				}
			});

			// finally show it
			alert.show();
		} else
			mHelper.launchPurchaseFlow(act, item.Id(), request, mPurchaseFinishedListener, payload);
	}

	public void consumeItem(InAppBillingItem item)
	{
		// check if this item is avaialble for being consumed
		Purchase purchase = inventory.getPurchase(item.Id());

		// if purchase is available to consume
		if (purchase != null)
			mHelper.consumeAsync(inventory.getPurchase(item.Id()), mConsumeFinishedListener);
		else
			// call consumption finished listener
			mConsumeFinishedListener.onConsumeFinished(null, new IabResult(IabHelper.IABHELPER_INVALID_CONSUMPTION, "Item not available for consumption!"));
	}

	public boolean handleActivityResult(int requestCode, int resultCode, Intent data)
	{
		return mHelper.handleActivityResult(requestCode, resultCode, data);
	}

	public InAppBillingItem getIabItemById(String id)
	{
		InAppBillingItem item = null;

		// loop through all iab items
		for (InAppBillingItem iab_sku : iabItems)
			if (id.equals(iab_sku.Id()))
			{
				item = iab_sku;

				break;
			}

		return item;
	}

	void alert(String message)
	{
		AlertDialog.Builder bld = new AlertDialog.Builder(act);
		bld.setMessage(message);
		bld.setNeutralButton("OK", null);
		bld.create().show();
	}

	public void dispose()
	{
		if (mHelper != null)
			mHelper.dispose();
		mHelper = null;
	}

}
